clear all
close all
clc

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   This code calculates global distribution of hiss wave amplitude and   %
%   wave normal angle inside the plasmapause by considering time history  %
%   of solar wind parameters(IMF Bz and solar wind speed) and AE index    %
%   as model inputs as well as variation in plasmapause location          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%-------------------------------------------------------------------------%
%    first, specify values of the 5 input parameters below
%    for calculating model outputs 
%    (i.e., wave amplidue, wave normal angle, plasmapause location) 
%-------------------------------------------------------------------------%
%    (example) if you want to estimate wave normal angle distribution 
%    of plasmaspheric hiss using SW model on non-equatorial region, 
%    input parameters should be set as follows:
%
%    n_input=3
%    hist_t=6
%    target_id=5
%    mlat_mode=1
%-------------------------------------------------------------------------%
n_input=1;        % specify value of total number of input parameters for hiss wave model
                  % "1" or "3" available 
                  % 1: AE index only for AEo or AE model
                  % 3: AE index, southward IMF Bz, solar wind speed(Vsw) for SW model
%-------------------------------------------------------------------------%
hist_t=0;         % specify value of time history (in hrs) of input 
                  % to include as model inputs for hiss wave model
                  % "0" or "6" available                   
                  % 0: use only current value of input 
                  % 6: use time history up to 6 hours 
                  % set "0" for AEo model
                  % set "6" for AE or SW model
%-------------------------------------------------------------------------%
target_id=4;      % which model output you want to calculate its global distribution
                  % "4" or "5" available
                  % 4: wave amplidue (Bw) model
                  % 5: wave normal angle (WNA) model
%-------------------------------------------------------------------------%
mlat_mode=1;      % which magnetic latitude zone of hiss distribution you want to calculate
                  % "1" or "2" available
                  % 1: eqautorial region, MLAT<=10.0
                  % 2: non-equatorial region, 10<MLAT<=20
%-------------------------------------------------------------------------%                  
lpp_hist_t=12;    % specify value of time history (in hrs) of AE index 
                  % to include as model inputs for plasmapause model
                  % only "12" available
                  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  loading solar wind parameters and geomagnetic indics with a 1-min      %
%  resolution, which will be used as model inputs for a geomagnetic storm %                                                               
%-------------------------------------------------------------------------%
%   1: interplanetary magnetic field Bx (nT)
%   2: interplanetary magnetic field By (nT)
%   3: interplanetary magnetic field Bz (nT)
%   4: interplanetary magnetic field Bt(total strength) (nT)
%   5: solar wind speed Vsw (km/s)
%   6: solar wind number density Nsw (/cc)
%   7: solar wind dynamic pressure Psw (nPa)
%   8: AE index (nT)
%   9: SYM-H (nT)
%   10: interplanetary electric field Ey in mV/m
%   11: Kp index
%   12: flow azimuthal angle phi(degrees), defined as cot(phi)=-Vsw,x/Vsw,y
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
par=load('./data/model_input.txt');   
bs=par(:,3);        % interplanetary magnetic field Bz in nT
vsw=par(:,5);       % solar wind speed Vsw in km/s
ae=par(:,8);        % AE index in nT

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%            making time series inputs for hiss wave model                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%------------ set input according to value of "n_input" ------------------%
if(n_input==1)
    par_in=ae;            % for AEo or AE model
elseif(n_input==3)
    par_in=[ae,bs,vsw];   % for SW model which includes AE index,IMF Bz,Vsw
end
%------------ make time series data for each input parameter -------------%
for id=1:n_input
for tid=1:hist_t+1
    for kk=1+60*hist_t:size(par_in,1)  
        if(tid==1)                     % current value of inputs
            input{id}(kk,tid)=par_in(kk,id);
        else                           % time history of inputs
            t1=kk-60*(tid-1);
            t2=kk-60*(tid-2)-1;
            input{id}(kk,tid)=nanmean(par_in(t1:t2,id)); 
        end
    end
end
end
%------- combine time series data for each input into one variable -------%
if(n_input==1)
    input_new=input{1};   % AE index 
elseif(n_input==3)
    input_new=[input{1},input{2},input{3}];  % AE, IMF Bz, Vsw
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    loading coefficeints of hiss model and mamixmum and minimum values   %
%    for normlaization in each bin                                        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(mlat_mode==1)
    ts2='mlat_below_10';
elseif(mlat_mode==2)
    ts2='mlat_above_10';
end

if(n_input==3)
    ts3='sw';
elseif(n_input==1)
    ts3='ae';
end 

if(target_id==4)
    ts4='bw';
elseif(target_id==5)
    ts4='wna';
end

%---------- loading model coefficeint in equation (2) --------------------%
load(['./data/hiss_model_coeff_for_',ts2,'/',ts4,'_',ts3,...
      '_model_coeff_hist_',num2str(hist_t),'hr.mat'])
coeff=final_model_coeff;
clear final_model_coeff

%---------- loading max and min value for normalization ------------------%
load(['./data/hiss_model_coeff_for_',ts2,'/',ts4,'_',ts3,...
      '_model_minmax_hist_',num2str(hist_t),'hr.mat'])

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   caluclating model output (i.e., wave amplitude or wave normal angle)  % 
%   for each time interval of a geomagnetic storm                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%-------- L-value grid from 2 to 9 with an interval of 1 Re---------------%
dl=1.0;
lmin=2.0;
lmax=9.0;

%-------- MLT grid from 0 to 22 with an interval of 2 hrs-----------------%
dmlt=2.0;
mltmin=0.0;
mltmax=24.0-dmlt;


for time=1:size(input_new,1)
l_id=0;
for lvalue=lmin:dl:lmax
    mlt_id=0;
    l_id=l_id+1;
    for mltvalue=mltmin:dmlt:mltmax
        mlt_id=mlt_id+1; 
        if(~isnan(coeff{l_id,mlt_id}))

            input_tmp=input_new(time,:);
            minp=range{l_id,mlt_id}(1,2:end);
            maxp=range{l_id,mlt_id}(2,2:end);
%--------------------- normalize model inputs ----------------------------%
            input_tmp_n = 2.*(input_tmp-minp)./(maxp-minp) - 1;
%-------------------------------------------------------------------------%
%   set the normalized input values of hiss model to be -1 or 1           %
%   when they are lower than -1 or larger than 1                          %
%-------------------------------------------------------------------------%
            rm_id=find(input_tmp_n<-1);
            input_tmp_n(rm_id)=-1;
            clear rm_id
            
            rm_id=find(input_tmp_n>1);
            input_tmp_n(rm_id)=1;
            clear rm_id minp maxp
%---------------- estimate model output using equation (2) ---------------% 
            tmp_n = coeff{l_id,mlt_id}(1)+...
                    sum(coeff{l_id,mlt_id}(2:end).*input_tmp_n'); 
%-------------------------------------------------------------------------%
%   set the normalized output value of hiss model to be -1 or 1           %
%   when they are lower than -1 or larger than 1                          %
%-------------------------------------------------------------------------%                
            if(tmp_n>1)
                tmp_n=1;
            elseif(tmp_n<-1)
                tmp_n=-1;
            end
%------------------ denormalize model output -----------------------------%    
            minp=range{l_id,mlt_id}(1,1);
            maxp=range{l_id,mlt_id}(2,1);    
            tmp=(tmp_n+1)*(maxp-minp)/2+minp;    
            model{time,1}(l_id,mlt_id)=tmp;
            clear minp maxp tmp_n tmp
        else
            model{time,1}(l_id,mlt_id)=NaN;
        end
        clear input_tmp input_tmp_n
    end  
end
end

for time=1:size(input_new,1)
    model{time,1}(:,end+1)=model{time,1}(:,1); 
end

clear par_in input_new input coeff

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    making time series input for plasmapause location (Lpp) model,       %
%    which only use time history of AE index as inputs                    %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
par_in=ae;            % AE index
for id=1:size(par_in,2)
for tid=1:lpp_hist_t+1
    for kk=1+60*lpp_hist_t:size(par_in,1)
        if(tid==1)                  % current value of inputs 
            lpp_input{id}(kk,tid)=par_in(kk,id);  
        else                        % time history of inputs
            t1=kk-60*(tid-1);
            t2=kk-60*(tid-2)-1;
            lpp_input{id}(kk,tid)=nanmean(par_in(t1:t2,id)); 
        end
    end
end
end
input_new=[lpp_input{1}];    % time series input

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%              loading coefficeints of Lpp model in equation (3)          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
load(['./data/lpp_model_coeff_hist_',num2str(lpp_hist_t),'hr.mat']);
lpp_model_coeff=coeff;
clear coeff

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  calculating plasmapause location for each time interval of 
%  a geomagnetic storm using mutiple liear regression model in equation (3) 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for time=1:size(input_new,1)
    mlt_id=0;
    for mltvalue=mltmin:dmlt:mltmax
        mlt_id=mlt_id+1; 
        if(~isnan(lpp_model_coeff{mlt_id}))
            tmp = lpp_model_coeff{mlt_id}(1)+...
             sum(lpp_model_coeff{mlt_id}(2:end).*input_new(time,:)');
            lpp_model{time,1}(mlt_id)=tmp;   % final Lpp model
        else
            lpp_model{time,1}(mlt_id)=NaN;
        end
        clear tmp
    end  
end

for time=1:size(input_new,1)
    lpp_model{time,1}(:,end+1)=lpp_model{time,1}(:,1); 
end

clear coeff lpp_model_coeff lpp_input input_new par_in

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% (1) linearly interpolate hiss between MLT sectors and L-values          %                                                        %
% (2) linearly interpolate lpp between MLT sectors                        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%------------------- original data grid of hiss model --------------------%
l0=[lmin:dl:lmax]; 
mltgrid=[mltmin:dmlt:mltmax+dmlt]; 
mlt0=(mltgrid*15).*(pi/180);
[th0,r0]=meshgrid(mlt0,l0);
[x0,y0]=pol2cart(th0,r0);
clear l0 th0 r0 mltgrid

%---------------- define new grid for interpolation ----------------------%
incl=dl/5;
l_int=[lmin:incl:lmax];
incmlt=1;     % in degree
mlt_int=[0:incmlt:360].*pi/180;    
[th_int,r_int]=meshgrid(mlt_int,l_int);
[x_int,y_int]=pol2cart(th_int,r_int);
clear th_int r_int

%------------------ interpolate hiss distribution ------------------------%
for time=1:size(model,1)
    model_int{time,1}=griddata(x0,y0,model{time,1},x_int,y_int);
end
clear x0 y0

%------------------ interpolate Lpp --------------------------------------%
for time=1:size(lpp_model,1)
    lpp_model_int{time,1}=interp1(mlt0,lpp_model{time,1},mlt_int);
end
clear mlt0

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     discard hiss waves estimated outside the model plasmapause          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
count=0;
for iter=24*60:15:length(model_int)
    count=count+1;
    amp{count,1}=model_int{iter};
    for ij=1:length(lpp_model_int{iter})
        rm=length(l_int(1):incl:lpp_model_int{iter}(ij));
        amp{count,1}(rm+1:end,ij)=NaN;   % final model output (wave amplitude or wave normal angle) 
        clear rm
    end
end
